<?php

namespace Modules\Cotacao\Modules\ListaPreco\Services;

use Doctrine\ORM\EntityManager;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class Sup01200MontagemListaPrecoService {
    protected $crudService;
    protected $entityManager;
    protected $environment;
    protected $connection;
    

    public function __construct($crudService, $entityManager, $environment, $connection) {
        $this->crudService       = $crudService;
        $this->entityManager     = $entityManager;
        $this->environment       = $environment;
        $this->connection        = $connection;
    }
    
    public function senhaOperador($cdoperador) {
        $query = $this->entityManager->getConnection()->prepare(
            "SELECT CDSENHAOPERWEB FROM OPERADOR WHERE CDOPERADOR = :CDCOMPRADOR");
        $query->execute(array(
            "CDCOMPRADOR" => $cdoperador
        ));
        $existe = $query->fetchAll();
        if(sizeof($existe) > 0)
            return $existe[0]["CDSENHAOPERWEB"];
        else
            return false;
    }
    public function validaData($row) {
       
        $query = "SELECT CDFILIAL, NRCOLEPRECO
                  FROM COLETAPR
                  WHERE DTINIVALCOL = TO_DATE(:P_DTINIVALCOL)
                  AND DTFINVALCOL <  TO_DATE(:P_DTFINVALCOL)
                  AND CDOPERADOR  = :P_CDOPERADOR";
        $params = array(
            "P_DTINIVALCOL" => $row[0]['P_DTINIVALCOL'],
            "P_DTFINVALCOL" => $row[0]["P_DTFINVALCOL"],
            "P_CDOPERADOR" =>  $row[0]['P_CDOPERADOR']
        );
    
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        return $stmt->fetchAll();
       
    }
    
    public function selectFilialNrcolepreco($row){
        $query = "SELECT CDFILIAL, NRCOLEPRECO 
                  FROM COLETAPR 
                  WHERE DTINIVALCOL = TO_DATE(:P_DTINIVALCOL) 
                  AND DTFINVALCOL < TO_DATE(:P_DTFINVALCOL) 
                  AND CDOPERADOR = :P_CDOPERADOR";
        $params = array(
            "P_DTINIVALCOL" => $row['P_DTINIVALCOL'],
            "P_DTFINVALCOL" => $row["P_DTFINVALCOL"],
            "P_CDOPERADOR"  => $row['P_CDOPERADOR']
        );
        
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        return $stmt->fetchAll();
    }
    
    public function selectCdregiao($row){
        $query = "SELECT CDREGIAO 
                  FROM REGIAO 
                  WHERE ((INSTR(:P_CDREGIAO, CDREGIAO) <> 0) OR :P_TCDREGIAO = 'T')";
        $params = array(
            "P_CDREGIAO" => $row["P_CDREGIAO"],
            "P_TCDREGIAO" => $row["P_TCDREGIAO"]
        );
        
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        return $stmt->fetchAll();
    }
    
    public function selectCdfilial($row){
        $query = "SELECT CDFILIAL 
                  FROM PREVLISTA 
                  WHERE (CDFILIAL = :P_CDFILIAL) 
                  AND (NRSEQPRELIST = :P_NRSEQPRELIST)";
        $params = array(
            "P_CDFILIAL" => $row['CDFILIAL'],
            "P_NRSEQPRELIST" => $row['NOVOCOD']
        );
        
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        return $stmt->fetchAll();
    }
    
    public function insertPrevlista($cdfilial, $novo_codigo, $regiao, $cdoperador, $row, $carregacot){
        $query = "INSERT INTO PREVLISTA (CDFILIAL, NRSEQPRELIST, DTCOMPPREV, CDOPERADOR, DTINIVALLIST, DTFINVALLIST, DSREGPREVLIS, IDATUCUSLIST, IDDISPLISFIL, CDPRODINILIST, CDPRODFINLIST, IDCOTSEMMOVLIST, IDCOINVALIDLIST, IDSTATUSPRLIST) VALUES (:P_CDFILIAL, :P_NRSEQPRELIST, SYSDATE, :P_CDOPERADOR, TO_DATE(:P_DTINIVALLIST), TO_DATE(:P_DTFINVALLIST), :P_DSREGPREVLIS, 'S', 'N', :P_CDPRODINI, :P_CDPRODFIN, :P_CARREGACOT, :P_COINCIDE, :P_STATUS)";
        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_NRSEQPRELIST" => $novo_codigo,
            "P_CDOPERADOR" => $cdoperador,
            "P_DTINIVALLIST" => $row["P_DTINIVALCOL"],
            "P_DTFINVALLIST" => $row["P_DTFINVALCOL"],
            "P_DSREGPREVLIS" => null,
            "P_CDPRODINI" => $row["P_CDPRODINI"],
            "P_CDPRODFIN" => $row["P_CDPRODFIN"] = self:: strPadRigthProd($row['P_CDPRODFIN']),
            "P_CARREGACOT" => $carregacot,
            "P_COINCIDE" => $row["P_COINCIDE"],
            "P_STATUS" => 'P'
        );
        
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
    }
    
    public function updateItemcole($cdfilial, $novo_codigo){
        $query = "UPDATE ITEMCOLE 
                 SET IDITMLISPREC = 'N',
                     IDPRECALT = 'N', 
                     IDITVALDIFLISTA = 'N', 
                     NRSEQPRELIST = NULL 
                 WHERE CDFILIAL = :P_CDFILIAL 
                 AND NRSEQPRELIST = :P_NRSEQPRELIST";
        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_NRSEQPRELIST" => $novo_codigo
        );
        
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
    }
    
    public function deletePrevlist($cdfilial, $novo_codigo){
        $query = "DELETE PREVLISTA 
                  WHERE CDFILIAL = :P_CDFILIAL 
                  AND NRSEQPRELIST = :P_NRSEQPRELIST";
        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_NRSEQPRELIST" => $novo_codigo
        );
        
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
    }
    
    public function deleteItmprevlist($cdfilial, $novo_codigo){
        $query = "DELETE FROM ITMPREVLIST 
                  WHERE CDFILIAL = :P_CDFILIAL 
                  AND NRSEQPRELIST = :P_NRSEQPRELIST ";
        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_NRSEQPRELIST" => $novo_codigo
        );
        
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
    }
    
    public function deleteItmprevlistNotExists($cdfilial, $novo_codigo){
        $query = "DELETE FROM ITMPREVLIST
                  WHERE NRSEQPRELIST = :P_NRSEQPRELIST 
                  AND CDFILIAL = :P_CDFILIAL
                  AND NOT EXISTS(
                      SELECT COLETAPR.CDFILIAL, COLETAPR.NRCOLEPRECO 
                      FROM COLETAPR
                      WHERE COLETAPR.IDDISMONTLIS <> 'N' 
                      AND COLETAPR.CDFILIAL = ITMPREVLIST.CDFILIAL 
                      AND COLETAPR.NRCOLEPRECO = ITMPREVLIST.NRCOLEPRECO
                  )";
        $params = array(
            "P_NRSEQPRELIST" => $novo_codigo,
            "P_CDFILIAL" => $cdfilial
        );
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
    }
    
    public function insertItmprevlist($cdfilial, $novo_codigo, $cdoperador, $regiao, $row, $coincideData){
        if($coincideData){
            $expressaoDataIni = "ITEMCOLE.DTINIVALITM = :P_DTINI";
            $expressaoDataFin = "ITEMCOLE.DTFINVALITM = :P_DTFIN";
        } else {
            $expressaoDataIni = "ITEMCOLE.DTINIVALITM <= :P_DTFIN";
            $expressaoDataFin = "ITEMCOLE.DTFINVALITM >= :P_DTINI";
            
        }
        $query = "INSERT INTO ITMPREVLIST ( CDFILIAL, NRSEQPRELIST, NRCOLEPRECO, CDFORNECED, CDREGIAO, CDPRODUTO, CDMARCA, IDITMSELLIS, CDFORMPGTO, VRPRECUNIIPL, VRPRECECOIPL, VRPRECOIPLPI ) ( SELECT ITEMCOLE.CDFILIAL, :P_NRSEQPRELIST, ITEMCOLE.NRCOLEPRECO, ITEMCOLE.CDFORNECED, ITEMCOLE.CDREGIAO, ITEMCOLE.CDPRODUTO, ITEMCOLE.CDMARCA, NVL(ITMPREVLIST.IDITMSELLIS,'N'), ITEMCOLE.CDFORMPGTO, ITEMCOLE.VRPRECUNIITM, ITEMCOLE.VRPRECECOITM, ITEMCOLE.VRPRECOITMPI FROM ITEMCOLE, COLETAPR, PRODUTO, ITMPREVLIST WHERE ( COLETAPR.CDFILIAL =:P_CDFILIAL ) AND ( COLETAPR.CDOPERADOR =:P_CDOPERADOR ) AND ( COLETAPR.IDTIPCOLREG IN ( 'G', 'W', 'E' ) OR :P_CARGCOT = 'T' ) AND ( COLETAPR.CDFILIAL = ITEMCOLE.CDFILIAL ) AND ( COLETAPR.NRCOLEPRECO = ITEMCOLE.NRCOLEPRECO ) AND ( COLETAPR.IDDISMONTLIS <> 'N' ) AND ( ITMPREVLIST.NRSEQPRELIST (+) =:P_NRSEQPRELIST ) AND ( ITMPREVLIST.CDFILIAL (+) = ITEMCOLE.CDFILIAL ) AND ( ITMPREVLIST.NRCOLEPRECO (+) = ITEMCOLE.NRCOLEPRECO ) AND ( ITMPREVLIST.CDFORNECED (+) = ITEMCOLE.CDFORNECED ) AND ( ITMPREVLIST.CDREGIAO (+) = ITEMCOLE.CDREGIAO ) AND ( ITMPREVLIST.CDPRODUTO (+) = ITEMCOLE.CDPRODUTO ) AND ( ITMPREVLIST.CDMARCA (+) = ITEMCOLE.CDMARCA ) AND ( ".$expressaoDataIni." ) AND ( ".$expressaoDataFin." ) AND ITEMCOLE.VRPRECUNIITM > 0 AND ( ITEMCOLE.CDREGIAO =:P_CDREGIAO ) AND NOT EXISTS ( SELECT CDFILIAL FROM ITMPREVLIST ITJA WHERE ITJA.CDFILIAL = ITMPREVLIST.CDFILIAL AND ITJA.NRSEQPRELIST = ITMPREVLIST.NRSEQPRELIST AND ITJA.NRCOLEPRECO = ITMPREVLIST.NRCOLEPRECO AND ITJA.CDREGIAO = ITMPREVLIST.CDREGIAO AND ITJA.CDFORNECED = ITMPREVLIST.CDFORNECED AND ITJA.CDPRODUTO = ITMPREVLIST.CDPRODUTO AND ITJA.CDMARCA = ITMPREVLIST.CDMARCA ) AND NOT EXISTS ( SELECT CDREGIAO FROM REFOREG WHERE ITEMCOLE.CDREGIAO = REFOREG.CDREGIAO AND ITEMCOLE.CDFORNECED = REFOREG.CDFORNECED ) AND ITEMCOLE.CDPRODUTO = PRODUTO.CDPRODUTO AND ( PRODUTO.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN OR :P_TPRODUTO = 'T' ) AND ( ITEMCOLE.CDREGIAO =:P_CDREGIAO ) AND ( EXISTS ( SELECT PRODCOMP.CDPRODUTO FROM PRODCOMP, PRODUTO PR WHERE PRODCOMP.CDPRODUTO = PR.CDPRODUTO AND PRODCOMP.CDOPERADOR =:P_CDOPERADOR AND ( PRODUTO.CDARVPROD LIKE PR.CDARVPROD || '%' ) ) ) )";
        $params = array(
            "P_NRSEQPRELIST" => $novo_codigo,
            "P_CDFILIAL"     => $cdfilial,
            "P_CDOPERADOR"   => $cdoperador,
            "P_CARGCOT"      => $row["P_CARREGACOT"],
            "P_DTFIN"        => $row["P_DTFINVALCOL"],
            "P_DTINI"        => $row["P_DTINIVALCOL"],
            "P_CDREGIAO"     => $regiao,
            "P_CDPRODINI"    => $row["P_CDPRODINI"],
            "P_CDPRODFIN"    => $row["P_CDPRODFIN"] = self:: strPadRigthProd($row['P_CDPRODFIN']),
            "P_TPRODUTO"     => $row["P_TPRODUTO"]
        );
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
    }
    private function strPadRigthProd($value){
	    return str_pad($value, 20, 'Z', STR_PAD_RIGHT);
	}
    
    public function selectContadorItmprevlist($cdfilial, $novo_codigo){
        $query = "SELECT COUNT(*) AS CONTADOR FROM ITMPREVLIST WHERE(NRSEQPRELIST = :P_NRSEQPRELIST) AND (CDFILIAL = :P_CDFILIAL)";
        $params = array(
            "P_NRSEQPRELIST" => $novo_codigo,
            "P_CDFILIAL" => $cdfilial
        );
        
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        return $stmt->fetchAll();
    }
    
    public function verificaCentral($cdfilial) {
       
        $query = "SELECT NVL(CDFILCENTCMP, CDFILIAL) AS CDFILCENTCMP
                    FROM FILIAL
                    WHERE CDFILIAL = :P_CDFILIAL";
        $params = array(
            "P_CDFILIAL" =>$cdfilial
        );
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        return $stmt->fetchAll();
       
    }
    
    public function updatePrevLista($cdfilial, $novo_codigo) {
       
        $query = "UPDATE PREVLISTA SET DTCOMPPREV = SYSDATE, IDDISPLISFIL = 'N', IDATUCUSLIST = 'S', IDPREVSELLIS = 'N', IDSTATUSPRLIST = DECODE(IDSTATUSPRLIST,:P_STATUS,IDSTATUSPRLIST,:P_STATUS), IDSTATUSANTLIST = DECODE(IDSTATUSPRLIST,:P_STATUS,IDSTATUSANTLIST,IDSTATUSPRLIST) WHERE ( CDFILIAL =:P_CDFILIAL ) AND ( NRSEQPRELIST =:P_NRSEQPRELIST )";
        $params = array(
            "P_STATUS"       =>     'P',
            "P_NRSEQPRELIST" =>     $novo_codigo,
            "P_CDFILIAL"     =>     $cdfilial
        );
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        return $stmt->fetchAll();
       
    }
    
}